/** @file zombie.h
 *  @author Mic
 *  @date 2004-2007
 */

#ifndef __ZOMBIE_H__
#define __ZOMBIE_H__

#include <windows.h>

#define	ASK 0L
#define TELL 1L

#define INPUT 0
#define OUTPUT 1

#define RDHDECL __fastcall
#define RDWDECL __fastcall
#define WRHDECL __fastcall
#define WRWDECL __fastcall

#define PALETTE_OFFSET		0xA4000
#define IOREGS_OFFSET		0xA5000
#define SPRITE_TABLE_OFFSET	0xA7000
#define BG2X_DIRTY			0xA8000
#define BG3X_DIRTY			0xA8004
#define IOREGS_ARM7_OFFSET	0xAC000

/** @brief CPU plugin messages */
#define CPU_IRQ_CALLBACK		0x000
#define CPU_IO_HANDLER			0x001
#define CPU_CYCLE_COUNT			0x002
#define CPU_REG_PC				0x003
#define CPU_REG_SP				0x004
#define CPU_BREAKPOINT			0x005
#define CPU_SPEED				0x006
#define CPU_INPB_CALLBACK		0x007
#define CPU_OUTPB_CALLBACK		0x008
#define CPU_CYCLE_SCALING		0x009
#define CPU_REG					0x00A
#define CPU_CYCLE_PTR			0x00B
#define CPU_ENABLE_IRQ			0x00C
#define CPU_DMA_CALLBACK		0x013
#define CPU_WAKE_UP				0x016

/** @brief MMU plugin messages */
#define MMU_READ_BYTE_CALLBACK	0x100
#define MMU_READ_WORD_CALLBACK	0x101
#define MMU_WRITE_BYTE_CALLBACK	0x102
#define MMU_WRITE_WORD_CALLBACK	0x103
#define MMU_TRANSLATE_ADDRESS_CALLBACK	0x104
#define MMU_MAP_PORT			0x105
#define MMU_READ_DWORD_CALLBACK	0x106
#define MMU_WRITE_DWORD_CALLBACK 0x107
#define MMU_MAP_MEMORY			0x108
#define MMU_GET_CALLBACK		0x109
#define MMU_IRQ_CALLBACK		0x10A
#define MMU_TIMER_OVERFLOW		0x10B
#define MMU_PAGE_TABLE			0x10C
#define MMU_TCM_REMAP			0x110
#define MMU_DTCM				0x111
#define MMU_READ_WORD_CALLBACKS	0x115
#define MMU_IRQ					0x116
#define MMU_IRQ_OVER			0x117
#define MMU_IRQ_OK				0x118
#define MMU_WRITE_WORD_CALLBACKS 0x119
#define MMU_VRAM_BANKS			0x11A
#define MMU_READ_DWORD_CALLBACKS 0x11B
#define MMU_WRITE_DWORD_CALLBACKS 0x11C
#define MMU_ITCM				0x11E
#define MMU_DISP_FIFO			0x130
#define MMU_HLE_TOUCH			0x140
#define MMU_PAGE_MASKS			0x141
#define MMU_READ_BYTE_CALLBACKS	0x148
#define MMU_WRITE_BYTE_CALLBACKS 0x149
#define MMU_TIMER_COUNTUP		0x14A
#define MMU_LZ77_UNPACK_ARM7	0x14B
#define MMU_LZ77_UNPACK_ARM9	0x14C
#define MMU_LZ77_UNPACK_WRAM_ARM7	0x17D
#define MMU_LZ77_UNPACK_WRAM_ARM9	0x17E
#define MMU_IRQ_CHECK			0x180
#define MMU_CRC16               0x181
#define MMU_GETSINETB           0x182
#define MMU_GETVOLTB            0x183
#define MMU_GETPITCHTB          0x184

#define GPU_CURRENT_SCANLINE	0x200
#define GPU_SCREEN_WIDTH		0x201
#define GPU_SCREEN_HEIGHT		0x202
#define GPU_FRAME_COMPLETED		0x203
#define GPU_VBLANK_LINES		0x204
#define GPU_HBLANK_PIXELS		0x205
#define GPU_MEMORY				0x206
#define GPU_NUMBER_COLORS		0x207
#define GPU_PALETTE				0x208
#define GPU_VBLANK_ACTION		0x209
#define GPU_HBLANK_ACTION		0x20A
#define GPU_SIZE_MULTIPLIER		0x20B
#define GPU_FRAME_RATE			0x20C
#define GPU_SET_CALLBACK		0x20D
#define GPU_3D_COMMAND			0x20E
#define GPU_GET_CALLBACK		0x20F
#define GPU_RENDERER_OK			0x210
#define GPU_FRAMEBUFFER_BGR24	0x220
#define GPU_FRAMEBUFFER_MAIN_BGR24 0x221
#define GPU_FRAMEBUFFER_SUB_BGR24  0x222
#define GPU_FRAMEBUFFER			0x223
#define GPU_FRAMEBUFFER_MAIN	0x224
#define GPU_FRAMEBUFFER_SUB		0x225
#define GPU_MAIN_OBJ			0x230
#define GPU_SUB_OBJ				0x231


#define APU_WRITE_CALLBACK		0x300
#define APU_MIX_CALLBACK		0x301
#define APU_PAUSE				0x302
#define	APU_SYNC				0x303


#define	ZOMBIE_WINDOW_HANDLE	0x500
#define ZOMBIE_ROM_BASE			0x501
#define ZOMBIE_KEYSTATE			0x502
#define ZOMBIE_ROM_SIZE			0x503
#define ZOMBIE_EVENT			0x504
#define ZOMBIE_HALT				0x508
#define ZOMBIE_TIME				0x509
#define ZOMBIE_DEBUG_MSG		0x511
#define ZOMBIE_LOG_EVENT		0x512
#define ZOMBIE_GBA_CART			0x533
#define ZOMBIE_ROM_PATH			0x534


#define TOUCHPAD_DATA			0x505

#define PLUGIN_NAME				0x600
#define LAST_ERROR				0x601
#define	PLUGIN_SETTING			0x602
#define PLUGIN_VERSION			0x603

#define INPUT_KEYDOWN			0x701
#define INPUT_KEYUP				0x702


#define WINDOW_HANDLE_MAIN		0
#define WINDOW_HANDLE_VIDEO		2

#define	EV_HBLANK_START			1
#define	EV_HBLANK_END			2
#define	EV_TIMER_OVERFLOW		6
#define EV_TIMER_COUNTUP		7
#define	EV_SUB_TIMER_OVERFLOW	10


#define KEY_UP					0
#define KEY_DOWN				1
#define KEY_LEFT				2
#define KEY_RIGHT				3
#define KEY_A					4
#define KEY_B					5
#define KEY_L					6
#define KEY_R					7
#define KEY_START				8
#define KEY_SELECT				9
#define KEY_X					10
#define KEY_Y					11


#define ARM7_FREQ 33554432

/**
 *  @brief VRAM bank type enumerator
 */
typedef enum vramBankType_t {
	MAIN_BG,			/**< Used for BG on main core */
	SUB_BG,				/**< Used for BG on sub core */
	MAIN_OBJ,			/**< Used for OBJ on main core */
	SUB_OBJ,			/**< Used for OBJ on sub core */
	MAIN_BGEXTPLTT,		/**< Used for extended BG palette on main core */
	SUB_BGEXTPLTT,		/**< Used for extended BG palette on sub core */
	MAIN_OBJEXTPLTT,	/**< Used for extended OBJ palette on main core */
	SUB_OBJEXTPLTT,		/**< Used for extended OBJ palette on sub core */
	TEX0,				/**< Used for texture 0 */
	TEX1,				/**< Used for texture 1 */
	TEX2,				/**< Used for texture 2 */
	TEX3,				/**< Used for texture 3 */
	TEXPLTT,			/**< Used for texture palette */
	ARM7,				/**< Used by ARM7 */
	LCDC,				/**< Used by LCDC */
	DISABLED			/**< Disabled */
};

/**
 *  @brief VRAM bank info struct
 */
typedef struct vram_bank
{
	int base;	/**< Base address 0..0x3FF */
	int size;	/**< Size in 16kB blocks */
	vramBankType_t type;	/**< Type of bank, see vramBankType_t  */
} vram_bank;


typedef struct palette_dib
{
	char *data;
	BITMAPINFOHEADER bih;
	RGBQUAD colors[256];
} palette_dib;

typedef struct touchpad
{
	int x,
		y,
		z,
		st;
} touchpad;

typedef int (__stdcall *cpu_init_callback)(void*);
typedef void(__stdcall *cpu_reset_callback)(void);
typedef void(__stdcall *cpu_irq_callback)(int);
typedef void(__stdcall *cpu_dma_callback)(char*,char*,int,int,int);
typedef void(__stdcall *cpu_set_io_handler_callback)(int,int,void*);
typedef int (__stdcall *cpu_inpb_callback)(int);
typedef void(__stdcall *cpu_outpb_callback)(int,int);
typedef int (__stdcall *cpu_get_callback)(int);
typedef void(__stdcall *cpu_set_callback)(unsigned int,unsigned int,unsigned int);
typedef unsigned int(__stdcall *cpu_execute_until_callback)(unsigned int);

typedef int          (__stdcall *mmu_init_callback)(void*);
typedef bool         (__stdcall *mmu_reset_callback)(void);
typedef unsigned char(RDHDECL *mmu_read_byte_callback)(int);
typedef short int    (RDHDECL *mmu_read_word_callback)(int);
typedef int          (RDWDECL *mmu_read_dword_callback)(int);
typedef void         (WRHDECL *mmu_write_byte_callback)(unsigned int,int);
typedef void         (WRHDECL *mmu_write_word_callback)(unsigned int,int);
typedef void         (WRWDECL *mmu_write_dword_callback)(unsigned int,int);
typedef int          (__stdcall *mmu_get_callback)(int);
typedef void         (__stdcall *mmu_set_callback)(unsigned int,unsigned int,unsigned int);
typedef void         (__stdcall *mmu_close_callback)(void);
typedef void         (__stdcall *mmu_irq_callback)(int);

typedef int (__stdcall *gpu_init_callback)(void*);
typedef void(__stdcall *gpu_reset_callback)(void);
typedef void(__stdcall *gpu_hblank_callback)(void);
typedef void(__stdcall *gpu_advance_line_callback)(void);
typedef void(__stdcall *gpu_set_callback)(int,int,int);
typedef int (__stdcall *gpu_get_callback)(int);
typedef void(__stdcall *gpu_close_callback)(void);

typedef int (__stdcall *apu_init_callback)(void*);
typedef void(__stdcall *apu_reset_callback)(void);
typedef int (__stdcall *apu_get_callback)(int);
typedef void(__stdcall *apu_set_callback)(unsigned int,unsigned int,unsigned int);
typedef void(__stdcall *apu_close_callback)(void);
typedef void(__stdcall *apu_write_callback)(unsigned int,unsigned int);
typedef void(__stdcall *apu_update_callback)(void);

typedef void*(__stdcall *zombie_callback)(int,int,void*,void*);
typedef void*(__stdcall *shell_callback)(int,int,DWORD,DWORD);

#define SHELL_WINDOW_HANDLE ZOMBIE_WINDOW_HANDLE

#endif
